@@ -4,6 +4,9 @@
 // `cargo bench unknown option --save-baseline`.
 // To pass args to criterion, use this form
 // `cargo bench --bench <name of the bench> -- --save-baseline <name>`.
+
+#![allow(clippy::exhaustive_structs)]
+
 use std::{
     collections::{BTreeMap, BTreeSet},
     convert::{TryFrom, TryInto},
@@ -16,7 +19,7 @@ use std::{
 use criterion::{criterion_group, criterion_main, Criterion};
 use event::StateEvent;
 use js_int::uint;
-use maplit::btreemap;
+use maplit::{btreemap, btreeset};
 use ruma_common::MilliSecondsSinceUnixEpoch;
 use ruma_events::{
     pdu::{EventHash, Pdu, RoomV3Pdu},
@@ -35,11 +38,11 @@ static SERVER_TIMESTAMP: AtomicU64 = AtomicU64::new(0);
 fn lexico_topo_sort(c: &mut Criterion) {
     c.bench_function("lexicographical topological sort", |b| {
         let graph = btreemap! {
-            event_id("l") => vec![event_id("o")],
-            event_id("m") => vec![event_id("n"), event_id("o")],
-            event_id("n") => vec![event_id("o")],
-            event_id("o") => vec![], // "o" has zero outgoing edges but 4 incoming edges
-            event_id("p") => vec![event_id("o")],
+            event_id("l") => btreeset![event_id("o")],
+            event_id("m") => btreeset![event_id("n"), event_id("o")],
+            event_id("n") => btreeset![event_id("o")],
+            event_id("o") => btreeset![], // "o" has zero outgoing edges but 4 incoming edges
+            event_id("p") => btreeset![event_id("o")],
         };
         b.iter(|| {
             let _ = StateResolution::lexicographical_topological_sort(&graph, |id| {
@@ -61,7 +64,7 @@ fn resolution_shallow_auth_chain(c: &mut Criterion) {
             let state_sets = vec![state_at_bob.clone(), state_at_charlie.clone()];
             let _ = match StateResolution::resolve::<StateEvent>(
                 &room_id(),
-                &RoomVersionId::Version2,
+                &RoomVersionId::Version6,
                 &state_sets,
                 state_sets
                     .iter()
@@ -82,10 +85,9 @@ fn resolution_shallow_auth_chain(c: &mut Criterion) {
 
 fn resolve_deeper_event_set(c: &mut Criterion) {
     c.bench_function("resolve state of 10 events 3 conflicting", |b| {
-        let init = INITIAL_EVENTS();
+        let mut inner = INITIAL_EVENTS();
         let ban = BAN_STATE_SET();
 
-        let mut inner = init;
         inner.extend(ban);
         let store = TestStore(inner.clone());
 
@@ -119,7 +121,7 @@ fn resolve_deeper_event_set(c: &mut Criterion) {
             let state_sets = vec![state_set_a.clone(), state_set_b.clone()];
             let _ = match StateResolution::resolve::<StateEvent>(
                 &room_id(),
-                &RoomVersionId::Version2,
+                &RoomVersionId::Version6,
                 &state_sets,
                 state_sets
                     .iter()
@@ -330,25 +332,11 @@ fn room_id() -> RoomId {
 }
 
 fn member_content_ban() -> JsonValue {
-    serde_json::to_value(MemberEventContent {
-        membership: MembershipState::Ban,
-        displayname: None,
-        avatar_url: None,
-        is_direct: None,
-        third_party_invite: None,
-    })
-    .unwrap()
+    serde_json::to_value(MemberEventContent::new(MembershipState::Ban)).unwrap()
 }
 
 fn member_content_join() -> JsonValue {
-    serde_json::to_value(MemberEventContent {
-        membership: MembershipState::Join,
-        displayname: None,
-        avatar_url: None,
-        is_direct: None,
-        third_party_invite: None,
-    })
-    .unwrap()
+    serde_json::to_value(MemberEventContent::new(MembershipState::Join)).unwrap()
 }
 
 pub fn to_pdu_event<S>(
@@ -363,6 +351,8 @@ pub fn to_pdu_event<S>(
 where
     S: AsRef<str>,
 {
+    // We don't care if the addition happens in order just that it is atomic
+    // (each event has its own value)
     let ts = SERVER_TIMESTAMP.fetch_add(1, SeqCst);
     let id = if id.contains('$') { id.to_string() } else { format!("${}:foo", id) };
     let auth_events = auth_events.iter().map(AsRef::as_ref).map(event_id).collect::<Vec<_>>();
@@ -385,7 +375,7 @@ where
             auth_events,
             prev_events,
             depth: uint!(0),
-            hashes: EventHash { sha256: "".into() },
+            hashes: EventHash::new(String::new()),
             signatures: btreemap! {},
         }),
     })
@@ -640,6 +630,8 @@ pub mod event {
                     _ => false,
                 },
                 Pdu::RoomV3Pdu(event) => event.state_key == Some("".into()),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -647,6 +639,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => serde_json::from_value(ev.content.clone()),
                 Pdu::RoomV3Pdu(ev) => serde_json::from_value(ev.content.clone()),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -654,6 +648,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.origin_server_ts,
                 Pdu::RoomV3Pdu(ev) => &ev.origin_server_ts,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -665,6 +661,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.sender,
                 Pdu::RoomV3Pdu(ev) => &ev.sender,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -672,6 +670,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.redacts.as_ref(),
                 Pdu::RoomV3Pdu(ev) => ev.redacts.as_ref(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -679,18 +679,24 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.room_id,
                 Pdu::RoomV3Pdu(ev) => &ev.room_id,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn kind(&self) -> EventType {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.kind.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.kind.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn state_key(&self) -> Option<String> {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.state_key.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.state_key.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -699,6 +705,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.origin.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.origin.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -706,6 +714,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.prev_events.iter().map(|(id, _)| id).cloned().collect(),
                 Pdu::RoomV3Pdu(ev) => ev.prev_events.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -713,6 +723,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.auth_events.iter().map(|(id, _)| id).cloned().collect(),
                 Pdu::RoomV3Pdu(ev) => ev.auth_events.to_vec(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -720,6 +732,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.content.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.content.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -727,6 +741,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.unsigned,
                 Pdu::RoomV3Pdu(ev) => &ev.unsigned,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -736,6 +752,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(_) => maplit::btreemap! {},
                 Pdu::RoomV3Pdu(ev) => ev.signatures.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -743,6 +761,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.hashes,
                 Pdu::RoomV3Pdu(ev) => &ev.hashes,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -750,6 +770,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.depth,
                 Pdu::RoomV3Pdu(ev) => &ev.depth,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -761,6 +783,8 @@ pub mod event {
                 Pdu::RoomV3Pdu(ev) => {
                     ev.kind == ev_type && ev.state_key.as_deref() == Some(state_key)
                 }
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -773,6 +797,8 @@ pub mod event {
             match self.rest {
                 Pdu::RoomV1Pdu(_) => RoomVersionId::Version1,
                 Pdu::RoomV3Pdu(_) => RoomVersionId::Version6,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
     }
