@@ -173,7 +173,7 @@ pub fn auth_check<E: Event>(
 
     // [synapse] checks for federation here
 
-    // 4. if type is m.room.aliases
+    // 4. If type is m.room.aliases
     if incoming_event.kind() == EventType::RoomAliases && room_version.special_case_aliases_auth {
         info!("starting m.room.aliases check");
 
@@ -304,7 +304,7 @@ pub fn valid_membership_change<E: Event>(
     auth_events: &StateMap<Arc<E>>,
 ) -> Result<bool> {
     let target_membership = serde_json::from_value::<MembershipState>(
-        content.get("membership").expect("we should test before that this field exists").clone(),
+        content.get("membership").expect("we test before that this field exists").clone(),
     )?;
 
     let third_party_invite = content
@@ -341,24 +341,12 @@ pub fn valid_membership_change<E: Event>(
     )?;
 
     let sender_power = power_levels.users.get(user_sender).map_or_else(
-        || {
-            if sender_membership != MembershipState::Join {
-                None
-            } else {
-                Some(&power_levels.users_default)
-            }
-        },
+        || (sender_membership == MembershipState::Join).then(|| &power_levels.users_default),
         // If it's okay, wrap with Some(_)
         Some,
     );
     let target_power = power_levels.users.get(&target_user_id).map_or_else(
-        || {
-            if target_membership != MembershipState::Join {
-                None
-            } else {
-                Some(&power_levels.users_default)
-            }
-        },
+        || (target_membership == MembershipState::Join).then(|| &power_levels.users_default),
         // If it's okay, wrap with Some(_)
         Some,
     );
@@ -493,7 +481,7 @@ pub fn can_send_event<E: Event>(event: &Arc<E>, auth_events: &StateMap<Arc<E>>)
     let event_type_power_level = get_send_level(&event.kind(), event.state_key(), ple);
     let user_level = get_user_power_level(event.sender(), auth_events);
 
-    debug!("{} ev_type {} usr {}", event.event_id().as_str(), event_type_power_level, user_level);
+    debug!("{} ev_type {} usr {}", event.event_id(), event_type_power_level, user_level);
 
     if user_level < event_type_power_level {
         return false;
@@ -531,7 +519,7 @@ pub fn check_power_levels<E: Event>(
     let current_content =
         serde_json::from_value::<PowerLevelsEventContent>(current_state.content()).unwrap();
 
-    // validation of users is done in Ruma, synapse for loops validating user_ids and integers here
+    // Validation of users is done in Ruma, synapse for loops validating user_ids and integers here
     info!("validation of power event finished");
 
     let user_level = get_user_power_level(power_event.sender(), auth_events);
@@ -739,21 +727,13 @@ pub fn get_user_power_level<E: Event>(user_id: &UserId, auth_events: &StateMap<A
             0 // TODO if this fails DB error?
         }
     } else {
-        // if no power level event found the creator gets 100 everyone else gets 0
+        // If no power level event found the creator gets 100 everyone else gets 0
         let key = (EventType::RoomCreate, "".into());
-        if let Some(create) = auth_events.get(&key) {
-            if let Ok(c) = serde_json::from_value::<CreateEventContent>(create.content()) {
-                if &c.creator == user_id {
-                    100
-                } else {
-                    0
-                }
-            } else {
-                0
-            }
-        } else {
-            0
-        }
+        auth_events
+            .get(&key)
+            .and_then(|create| serde_json::from_value::<CreateEventContent>(create.content()).ok())
+            .and_then(|create| (create.creator == *user_id).then(|| 100))
+            .unwrap_or_default()
     }
 }
 
@@ -764,7 +744,6 @@ pub fn get_send_level<E: Event>(
     state_key: Option<String>,
     power_lvl: Option<&Arc<E>>,
 ) -> i64 {
-    debug!("{:?} {:?}", e_type, state_key);
     power_lvl
         .and_then(|ple| {
             serde_json::from_value::<PowerLevelsEventContent>(ple.content())
@@ -808,9 +787,10 @@ pub fn verify_third_party_invite<E: Event>(
     tp_id: &ThirdPartyInvite,
     current_third_party_invite: Option<Arc<E>>,
 ) -> bool {
-    // 1. check for user being banned happens before this is called
+    // 1. Check for user being banned happens before this is called
     // checking for mxid and token keys is done by ruma when deserializing
 
+    // The state key must match the invitee
     if user_state_key != Some(tp_id.signed.mxid.as_str()) {
         return false;
     }
