@@ -33,6 +33,7 @@ pub type StateMap<T> = BTreeMap<(EventType, String), T>;
 pub type EventMap<T> = BTreeMap<EventId, T>;
 
 #[derive(Default)]
+#[allow(clippy::exhaustive_structs)]
 pub struct StateResolution;
 
 impl StateResolution {
@@ -58,7 +59,7 @@ impl StateResolution {
     ) -> Result<StateMap<EventId>> {
         info!("State resolution starting");
 
-        // split non-conflicting and conflicting state
+        // Split non-conflicting and conflicting state
         let (clean, conflicting) = StateResolution::separate(state_sets);
 
         info!("non conflicting {:?}", clean.len());
@@ -70,35 +71,36 @@ impl StateResolution {
 
         info!("{} conflicting events", conflicting.len());
 
-        // the set of auth events that are not common across server forks
+        // The set of auth events that are not common across server forks
         let mut auth_diff = StateResolution::get_auth_chain_diff(room_id, &auth_events)?;
 
         debug!("auth diff size {:?}", auth_diff);
 
-        // add the auth_diff to conflicting now we have a full set of conflicting events
+        // Add the auth_diff to conflicting now we have a full set of conflicting events
         auth_diff.extend(conflicting.values().cloned().flatten());
-        let mut all_conflicted =
-            auth_diff.into_iter().collect::<BTreeSet<_>>().into_iter().collect::<Vec<_>>();
+
+        // `all_conflicted` contains unique items
+        // synapse says `full_set = {eid for eid in full_conflicted_set if eid in event_map}`
+        //
+        // Don't honor events we cannot "verify"
+        // TODO: BTreeSet::retain() when stable 1.53
+        let all_conflicted =
+            auth_diff.into_iter().filter(|id| event_map.contains_key(id)).collect::<BTreeSet<_>>();
 
         info!("full conflicted set is {} events", all_conflicted.len());
 
-        // we used to check that all events are events from the correct room
+        // We used to check that all events are events from the correct room
         // this is now a check the caller of `resolve` must make.
 
-        // synapse says `full_set = {eid for eid in full_conflicted_set if eid in event_map}`
-        //
-        // don't honor events we cannot "verify"
-        all_conflicted.retain(|id| event_map.contains_key(id));
-
-        // get only the control events with a state_key: "" or ban/kick event (sender != state_key)
+        // Get only the control events with a state_key: "" or ban/kick event (sender != state_key)
         let control_events = all_conflicted
             .iter()
             .filter(|id| is_power_event_id(id, event_map))
             .cloned()
             .collect::<Vec<_>>();
 
-        // sort the control events based on power_level/clock/event_id and outgoing/incoming edges
-        let mut sorted_control_levels = StateResolution::reverse_topological_power_sort(
+        // Sort the control events based on power_level/clock/event_id and outgoing/incoming edges
+        let sorted_control_levels = StateResolution::reverse_topological_power_sort(
             room_id,
             &control_events,
             event_map,
@@ -108,7 +110,7 @@ impl StateResolution {
         debug!("SRTD {:?}", sorted_control_levels);
 
         let room_version = RoomVersion::new(room_version)?;
-        // sequentially auth check each control event.
+        // Sequentially auth check each control event.
         let resolved_control = StateResolution::iterative_auth_check(
             room_id,
             &room_version,
@@ -117,15 +119,11 @@ impl StateResolution {
             event_map,
         )?;
 
-        debug!(
-            "AUTHED {:?}",
-            resolved_control.iter().map(|(key, id)| (key, id.to_string())).collect::<Vec<_>>()
-        );
+        debug!("AUTHED {:?}", resolved_control.iter().collect::<Vec<_>>());
 
         // At this point the control_events have been resolved we now have to
         // sort the remaining events using the mainline of the resolved power level.
-        sorted_control_levels.dedup();
-        let deduped_power_ev = sorted_control_levels;
+        let deduped_power_ev = sorted_control_levels.into_iter().collect::<BTreeSet<_>>();
 
         // This removes the control events that passed auth and more importantly those that failed
         // auth
@@ -135,7 +133,7 @@ impl StateResolution {
             .cloned()
             .collect::<Vec<_>>();
 
-        debug!("LEFT {:?}", events_to_resolve.iter().map(ToString::to_string).collect::<Vec<_>>());
+        debug!("LEFT {:?}", events_to_resolve.iter().collect::<Vec<_>>());
 
         // This "epochs" power level event
         let power_event = resolved_control.get(&(EventType::RoomPowerLevels, "".into()));
@@ -145,10 +143,7 @@ impl StateResolution {
         let sorted_left_events =
             StateResolution::mainline_sort(room_id, &events_to_resolve, power_event, event_map);
 
-        debug!(
-            "SORTED LEFT {:?}",
-            sorted_left_events.iter().map(ToString::to_string).collect::<Vec<_>>()
-        );
+        debug!("SORTED LEFT {:?}", sorted_left_events.iter().collect::<Vec<_>>());
 
         let mut resolved_state = StateResolution::iterative_auth_check(
             room_id,
@@ -158,7 +153,7 @@ impl StateResolution {
             event_map,
         )?;
 
-        // add unconflicted state to the resolved state
+        // Add unconflicted state to the resolved state
         // We priorities the unconflicting state
         resolved_state.extend(clean);
         Ok(resolved_state)
@@ -175,21 +170,18 @@ impl StateResolution {
     ) -> (StateMap<EventId>, StateMap<Vec<EventId>>) {
         use itertools::Itertools;
 
-        info!("seperating {} sets of events into conflicted/unconflicted", state_sets.len());
+        info!("separating {} sets of events into conflicted/unconflicted", state_sets.len());
 
         let mut unconflicted_state = StateMap::new();
         let mut conflicted_state = StateMap::new();
 
-        for key in state_sets.iter().flat_map(|map| map.keys()).dedup() {
+        for key in state_sets.iter().flat_map(|map| map.keys()).unique() {
             let mut event_ids =
-                state_sets.iter().map(|state_set| state_set.get(key)).dedup().collect::<Vec<_>>();
+                state_sets.iter().map(|state_set| state_set.get(key)).unique().collect::<Vec<_>>();
 
             if event_ids.len() == 1 {
-                if let Some(Some(id)) = event_ids.pop() {
-                    unconflicted_state.insert(key.clone(), id.clone());
-                } else {
-                    panic!()
-                }
+                let id = event_ids.remove(0).expect("unconflicting `EventId` is not None");
+                unconflicted_state.insert(key.clone(), id.clone());
             } else {
                 conflicted_state.insert(
                     key.clone(),
@@ -205,9 +197,7 @@ impl StateResolution {
     pub fn get_auth_chain_diff(
         _room_id: &RoomId,
         auth_event_ids: &[Vec<EventId>],
-    ) -> Result<Vec<EventId>> {
-        use itertools::Itertools;
-
+    ) -> Result<BTreeSet<EventId>> {
         let mut chains = vec![];
 
         for ids in auth_event_ids {
@@ -221,9 +211,9 @@ impl StateResolution {
             let rest = chains.iter().skip(1).flatten().cloned().collect();
             let common = chain.intersection(&rest).collect::<Vec<_>>();
 
-            Ok(chains.into_iter().flatten().filter(|id| !common.contains(&id)).dedup().collect())
+            Ok(chains.into_iter().flatten().filter(|id| !common.contains(&id)).collect())
         } else {
-            Ok(vec![])
+            Ok(btreeset![])
         }
     }
 
@@ -238,7 +228,7 @@ impl StateResolution {
         room_id: &RoomId,
         events_to_sort: &[EventId],
         event_map: &mut EventMap<Arc<E>>,
-        auth_diff: &[EventId],
+        auth_diff: &BTreeSet<EventId>,
     ) -> Vec<EventId> {
         debug!("reverse topological sort of power events");
 
@@ -253,11 +243,11 @@ impl StateResolution {
             // tasks can make progress
         }
 
-        // this is used in the `key_fn` passed to the lexico_topo_sort fn
+        // This is used in the `key_fn` passed to the lexico_topo_sort fn
         let mut event_to_pl = BTreeMap::new();
         for event_id in graph.keys() {
             let pl = StateResolution::get_power_level_for_sender(room_id, event_id, event_map);
-            info!("{} power level {}", event_id.to_string(), pl);
+            info!("{} power level {}", event_id, pl);
 
             event_to_pl.insert(event_id.clone(), pl);
 
@@ -267,7 +257,6 @@ impl StateResolution {
         }
 
         StateResolution::lexicographical_topological_sort(&graph, |event_id| {
-            // debug!("{:?}", event_map.get(event_id).unwrap().origin_server_ts());
             let ev = event_map.get(event_id).unwrap();
             let pl = event_to_pl.get(event_id).unwrap();
 
@@ -284,7 +273,7 @@ impl StateResolution {
     /// `key_fn` is used as a tie breaker. The tie breaker happens based on
     /// power level, age, and event_id.
     pub fn lexicographical_topological_sort<F>(
-        graph: &BTreeMap<EventId, Vec<EventId>>,
+        graph: &BTreeMap<EventId, BTreeSet<EventId>>,
         key_fn: F,
     ) -> Vec<EventId>
     where
@@ -301,8 +290,7 @@ impl StateResolution {
         // TODO make the BTreeSet conversion cleaner ??
         // outdegree_map is an event referring to the events before it, the
         // more outdegree's the more recent the event.
-        let mut outdegree_map: BTreeMap<EventId, BTreeSet<EventId>> =
-            graph.iter().map(|(k, v)| (k.clone(), v.iter().cloned().collect())).collect();
+        let mut outdegree_map = graph.clone();
 
         // The number of events that depend on the given event (the eventId key)
         let mut reverse_graph = BTreeMap::new();
@@ -312,7 +300,7 @@ impl StateResolution {
 
         for (node, edges) in graph.iter() {
             if edges.is_empty() {
-                // the `Reverse` is because rusts `BinaryHeap` sorts largest -> smallest we need
+                // The `Reverse` is because rusts `BinaryHeap` sorts largest -> smallest we need
                 // smallest -> largest
                 zero_outdegree.push(Reverse((key_fn(node), node)));
             }
@@ -325,16 +313,16 @@ impl StateResolution {
 
         let mut heap = BinaryHeap::from(zero_outdegree);
 
-        // we remove the oldest node (most incoming edges) and check against all other
+        // We remove the oldest node (most incoming edges) and check against all other
         let mut sorted = vec![];
-        // destructure the `Reverse` and take the smallest `node` each time
+        // Destructure the `Reverse` and take the smallest `node` each time
         while let Some(Reverse((_, node))) = heap.pop() {
             let node: &EventId = node;
             for parent in reverse_graph.get(node).unwrap() {
-                // the number of outgoing edges this node has
+                // The number of outgoing edges this node has
                 let out = outdegree_map.get_mut(parent).unwrap();
 
-                // only push on the heap once older events have been cleared
+                // Only push on the heap once older events have been cleared
                 out.remove(node);
                 if out.is_empty() {
                     heap.push(Reverse((key_fn(parent), parent)));
@@ -354,7 +342,7 @@ impl StateResolution {
         event_id: &EventId,
         event_map: &mut EventMap<Arc<E>>,
     ) -> i64 {
-        info!("fetch event ({}) senders power level", event_id.to_string());
+        info!("fetch event ({}) senders power level", event_id);
 
         let event = StateResolution::get_or_load_event(room_id, event_id, event_map);
         let mut pl = None;
@@ -379,7 +367,7 @@ impl StateResolution {
         {
             if let Ok(ev) = event {
                 if let Some(user) = content.users.get(ev.sender()) {
-                    debug!("found {} at power_level {}", ev.sender().as_str(), user);
+                    debug!("found {} at power_level {}", ev.sender(), user);
                     return (*user).into();
                 }
             }
@@ -408,10 +396,7 @@ impl StateResolution {
     ) -> Result<StateMap<EventId>> {
         info!("starting iterative auth check");
 
-        debug!(
-            "performing auth checks on {:?}",
-            events_to_check.iter().map(ToString::to_string).collect::<Vec<_>>()
-        );
+        debug!("performing auth checks on {:?}", events_to_check.iter().collect::<Vec<_>>());
 
         let mut resolved_state = unconflicted_state.clone();
 
@@ -455,7 +440,7 @@ impl StateResolution {
                 }
             }
 
-            debug!("event to check {:?}", event.event_id().as_str());
+            debug!("event to check {:?}", event.event_id());
 
             let most_recent_prev_event = event
                 .prev_events()
@@ -466,11 +451,10 @@ impl StateResolution {
             // The key for this is (eventType + a state_key of the signed token not sender) so
             // search for it
             let current_third_party = auth_events.iter().find_map(|(_, pdu)| {
-                if pdu.kind() == EventType::RoomThirdPartyInvite {
-                    Some(pdu.clone()) // TODO no clone, auth_events is borrowed while moved
-                } else {
-                    None
-                }
+                (pdu.kind() == EventType::RoomThirdPartyInvite).then(|| {
+                    // TODO no clone, auth_events is borrowed while moved
+                    pdu.clone()
+                })
             });
 
             if auth_check(
@@ -484,7 +468,7 @@ impl StateResolution {
                 resolved_state.insert((event.kind(), state_key), event_id.clone());
             } else {
                 // synapse passes here on AuthError. We do not add this event to resolved_state.
-                warn!("event {} failed the authentication check", event_id.to_string());
+                warn!("event {} failed the authentication check", event_id);
             }
 
             // TODO: if these functions are ever made async here
@@ -566,7 +550,7 @@ impl StateResolution {
             // tasks can make progress
         }
 
-        // sort the event_ids by their depth, timestamp and EventId
+        // Sort the event_ids by their depth, timestamp and EventId
         // unwrap is OK order map and sort_event_ids are from to_sort (the same Vec)
         let mut sort_event_ids = order_map.keys().map(|&k| k.clone()).collect::<Vec<_>>();
         sort_event_ids.sort_by_key(|sort_id| order_map.get(sort_id).unwrap());
@@ -583,17 +567,15 @@ impl StateResolution {
         event_map: &mut EventMap<Arc<E>>,
     ) -> Result<usize> {
         while let Some(sort_ev) = event {
-            debug!("mainline event_id {}", sort_ev.event_id().to_string());
+            debug!("mainline event_id {}", sort_ev.event_id());
             let id = &sort_ev.event_id();
             if let Some(depth) = mainline_map.get(id) {
                 return Ok(*depth);
             }
 
-            // dbg!(&sort_ev);
             let auth_events = &sort_ev.auth_events();
             event = None;
             for aid in auth_events {
-                // dbg!(&aid);
                 let aev = StateResolution::get_or_load_event(room_id, aid, event_map)?;
                 if is_type_and_key(&aev, EventType::RoomPowerLevels, "") {
                     event = Some(aev);
@@ -607,28 +589,28 @@ impl StateResolution {
 
     fn add_event_and_auth_chain_to_graph<E: Event>(
         room_id: &RoomId,
-        graph: &mut BTreeMap<EventId, Vec<EventId>>,
+        graph: &mut BTreeMap<EventId, BTreeSet<EventId>>,
         event_id: &EventId,
         event_map: &mut EventMap<Arc<E>>,
-        auth_diff: &[EventId],
+        auth_diff: &BTreeSet<EventId>,
     ) {
         let mut state = vec![event_id.clone()];
         while !state.is_empty() {
-            // we just checked if it was empty so unwrap is fine
+            // We just checked if it was empty so unwrap is fine
             let eid = state.pop().unwrap();
-            graph.entry(eid.clone()).or_insert_with(Vec::new);
-            // prefer the store to event as the store filters dedups the events
-            // otherwise it seems we can loop forever
-            for aid in
-                &StateResolution::get_or_load_event(room_id, &eid, event_map).unwrap().auth_events()
+            graph.entry(eid.clone()).or_insert(btreeset![]);
+            // Prefer the store to event as the store filters dedups the events
+            for aid in &StateResolution::get_or_load_event(room_id, &eid, event_map)
+                .map(|ev| ev.auth_events())
+                .unwrap_or_default()
             {
                 if auth_diff.contains(aid) {
                     if !graph.contains_key(aid) {
                         state.push(aid.clone());
                     }
 
-                    // we just inserted this at the start of the while loop
-                    graph.get_mut(&eid).unwrap().push(aid.clone());
+                    // We just inserted this at the start of the while loop
+                    graph.get_mut(&eid).unwrap().insert(aid.clone());
                 }
             }
         }
