@@ -1,4 +1,4 @@
-#![allow(clippy::or_fun_call, clippy::expect_fun_call, dead_code)]
+#![allow(dead_code)]
 
 use std::{
     collections::{BTreeMap, BTreeSet},
@@ -10,7 +10,7 @@ use std::{
 };
 
 use js_int::uint;
-use maplit::btreemap;
+use maplit::{btreemap, btreeset};
 use ruma_common::MilliSecondsSinceUnixEpoch;
 use ruma_events::{
     pdu::{EventHash, Pdu, RoomV3Pdu},
@@ -37,9 +37,8 @@ pub fn do_check(
     edges: Vec<Vec<EventId>>,
     expected_state_ids: Vec<EventId>,
 ) {
-    // to activate logging use `RUST_LOG=debug cargo t`
-    let _ = LOGGER
-        .call_once(|| tracer::fmt().with_env_filter(tracer::EnvFilter::from_default_env()).init());
+    // To activate logging use `RUST_LOG=debug cargo t`
+    // The logger is initialized in the `INITIAL_EVENTS` function.
 
     let init_events = INITIAL_EVENTS();
 
@@ -49,26 +48,26 @@ pub fn do_check(
 
     // This will be lexi_topo_sorted for resolution
     let mut graph = BTreeMap::new();
-    // this is the same as in `resolve` event_id -> StateEvent
+    // This is the same as in `resolve` event_id -> StateEvent
     let mut fake_event_map = BTreeMap::new();
 
-    // create the DB of events that led up to this point
+    // Create the DB of events that led up to this point
     // TODO maybe clean up some of these clones it is just tests but...
     for ev in init_events.values().chain(events) {
-        graph.insert(ev.event_id().clone(), vec![]);
+        graph.insert(ev.event_id().clone(), btreeset![]);
         fake_event_map.insert(ev.event_id().clone(), ev.clone());
     }
 
     for pair in INITIAL_EDGES().windows(2) {
         if let [a, b] = &pair {
-            graph.entry(a.clone()).or_insert(vec![]).push(b.clone());
+            graph.entry(a.clone()).or_insert_with(BTreeSet::new).insert(b.clone());
         }
     }
 
     for edge_list in edges {
         for pair in edge_list.windows(2) {
             if let [a, b] = &pair {
-                graph.entry(a.clone()).or_insert(vec![]).push(b.clone());
+                graph.entry(a.clone()).or_insert_with(BTreeSet::new).insert(b.clone());
             }
         }
     }
@@ -78,7 +77,7 @@ pub fn do_check(
     // event_id -> StateMap<EventId>
     let mut state_at_event: BTreeMap<EventId, StateMap<EventId>> = BTreeMap::new();
 
-    // resolve the current state and add it to the state_at_event map then continue
+    // Resolve the current state and add it to the state_at_event map then continue
     // on in "time"
     for node in StateResolution::lexicographical_topological_sort(&graph, |id| {
         (0, MilliSecondsSinceUnixEpoch(uint!(0)), id.clone())
@@ -91,7 +90,7 @@ pub fn do_check(
         let state_before: StateMap<EventId> = if prev_events.is_empty() {
             BTreeMap::new()
         } else if prev_events.len() == 1 {
-            state_at_event.get(&prev_events[0]).unwrap().clone()
+            state_at_event.get(prev_events.iter().next().unwrap()).unwrap().clone()
         } else {
             let state_sets = prev_events
                 .iter()
@@ -161,10 +160,10 @@ pub fn do_check(
             Some(&e.state_key()),
             e.content(),
             &auth_events,
-            prev_events,
+            &prev_events.iter().cloned().collect::<Vec<_>>(),
         );
 
-        // we have to update our store, an actual user of this lib would
+        // We have to update our store, an actual user of this lib would
         // be giving us state from a DB.
         store.0.insert(ev_id.clone(), event.clone());
 
@@ -174,11 +173,13 @@ pub fn do_check(
 
     let mut expected_state = StateMap::new();
     for node in expected_state_ids {
-        let ev = event_map.get(&node).expect(&format!(
-            "{} not found in {:?}",
-            node.to_string(),
-            event_map.keys().map(ToString::to_string).collect::<Vec<_>>(),
-        ));
+        let ev = event_map.get(&node).unwrap_or_else(|| {
+            panic!(
+                "{} not found in {:?}",
+                node.to_string(),
+                event_map.keys().map(ToString::to_string).collect::<Vec<_>>()
+            )
+        });
 
         let key = (ev.kind(), ev.state_key());
 
@@ -197,7 +198,7 @@ pub fn do_check(
                 // Filter out the dummy messages events.
                 // These act as points in time where there should be a known state to
                 // test against.
-                && k != &&(EventType::RoomMessage, "dummy".to_string())
+                && **k != (EventType::RoomMessage, "dummy".to_string())
         })
         .map(|(k, v)| (k.clone(), v.clone()))
         .collect::<StateMap<EventId>>();
@@ -205,11 +206,11 @@ pub fn do_check(
     assert_eq!(expected_state, end_state);
 }
 
+#[allow(clippy::exhaustive_structs)]
 pub struct TestStore<E: Event>(pub BTreeMap<EventId, Arc<E>>);
 
-#[allow(unused)]
 impl<E: Event> TestStore<E> {
-    pub fn get_event(&self, room_id: &RoomId, event_id: &EventId) -> Result<Arc<E>> {
+    pub fn get_event(&self, _: &RoomId, event_id: &EventId) -> Result<Arc<E>> {
         self.0
             .get(event_id)
             .map(Arc::clone)
@@ -335,7 +336,7 @@ pub fn to_init_pdu_event(
             auth_events: vec![],
             prev_events: vec![],
             depth: uint!(0),
-            hashes: EventHash { sha256: "".into() },
+            hashes: EventHash::new("".to_owned()),
             signatures: btreemap! {},
         }),
     })
@@ -375,7 +376,7 @@ where
             auth_events,
             prev_events,
             depth: uint!(0),
-            hashes: EventHash { sha256: "".into() },
+            hashes: EventHash::new("".to_owned()),
             signatures: btreemap! {},
         }),
     })
@@ -543,6 +544,7 @@ pub mod event {
     }
 
     #[derive(Clone, Debug, Deserialize, Serialize)]
+    #[allow(clippy::exhaustive_structs)]
     pub struct StateEvent {
         pub event_id: EventId,
         #[serde(flatten)]
@@ -590,18 +592,24 @@ pub mod event {
                     _ => false,
                 },
                 Pdu::RoomV3Pdu(event) => event.state_key == Some("".into()),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn deserialize_content<C: serde::de::DeserializeOwned>(&self) -> serde_json::Result<C> {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => serde_json::from_value(ev.content.clone()),
                 Pdu::RoomV3Pdu(ev) => serde_json::from_value(ev.content.clone()),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn origin_server_ts(&self) -> &MilliSecondsSinceUnixEpoch {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.origin_server_ts,
                 Pdu::RoomV3Pdu(ev) => &ev.origin_server_ts,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn event_id(&self) -> &EventId {
@@ -612,6 +620,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.sender,
                 Pdu::RoomV3Pdu(ev) => &ev.sender,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -619,6 +629,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.redacts.as_ref(),
                 Pdu::RoomV3Pdu(ev) => ev.redacts.as_ref(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -626,18 +638,24 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.room_id,
                 Pdu::RoomV3Pdu(ev) => &ev.room_id,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn kind(&self) -> EventType {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.kind.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.kind.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
         pub fn state_key(&self) -> String {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.state_key.clone().unwrap(),
                 Pdu::RoomV3Pdu(ev) => ev.state_key.clone().unwrap(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -646,6 +664,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.origin.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.origin.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -653,6 +673,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.prev_events.iter().map(|(id, _)| id).cloned().collect(),
                 Pdu::RoomV3Pdu(ev) => ev.prev_events.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -660,6 +682,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.auth_events.iter().map(|(id, _)| id).cloned().collect(),
                 Pdu::RoomV3Pdu(ev) => ev.auth_events.to_vec(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -667,6 +691,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => ev.content.clone(),
                 Pdu::RoomV3Pdu(ev) => ev.content.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -674,6 +700,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.unsigned,
                 Pdu::RoomV3Pdu(ev) => &ev.unsigned,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -683,6 +711,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(_) => maplit::btreemap! {},
                 Pdu::RoomV3Pdu(ev) => ev.signatures.clone(),
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -690,6 +720,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.hashes,
                 Pdu::RoomV3Pdu(ev) => &ev.hashes,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -697,6 +729,8 @@ pub mod event {
             match &self.rest {
                 Pdu::RoomV1Pdu(ev) => &ev.depth,
                 Pdu::RoomV3Pdu(ev) => &ev.depth,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -708,6 +742,8 @@ pub mod event {
                 Pdu::RoomV3Pdu(ev) => {
                     ev.kind == ev_type && ev.state_key.as_deref() == Some(state_key)
                 }
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
 
@@ -720,6 +756,8 @@ pub mod event {
             match self.rest {
                 Pdu::RoomV1Pdu(_) => RoomVersionId::Version1,
                 Pdu::RoomV3Pdu(_) => RoomVersionId::Version6,
+                #[cfg(not(feature = "unstable-exhaustive-types"))]
+                _ => unreachable!("new PDU version"),
             }
         }
     }
